package Catalyst::Runtime;

use strict;
use warnings;

BEGIN { require 5.008001; }

# Remember to update this in Catalyst as well!

our $VERSION='5.7015';

=head1 NAME

Catalyst::Runtime - Catalyst  Runtime version

=head1 SYNOPSIS

See L<Catalyst>.

=head1 DESCRIPTION

This is the primary class for the Catalyst-Runtime distribution, version 5.70.

=head1 AUTHOR

The Catalyst Core Team - see http://catalyst.perl.org/

=head1 COPYRIGHT

This program is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut

1;
